function[fOut, rho, ux, uy] = lattboltz(fIn, tau, u0) % Lattice Boltzmann

[m,n,~] = size(fIn);
t = [4/9 1/9 1/9 1/9 1/9 1/36 1/36 1/36 1/36];
c = [[0,0];[0,1];[-1,0];[0,-1];[1,0];[-1,1];[-1,-1];[1,-1];[1,1]];

fOut = zeros(m,n,9); rho = zeros(m,n);
ux = zeros(m,n); uy = zeros(m,n); fEq = zeros(m,n,9);

for i = 1 : 9 % Calculer les fEq aux bords
   fEq(:,1,i) = t(i); % Gauche
   fEq(:,n,i) = t(i); % Droite
   fEq(1,2:n-1,i) = t(i)*(1 + u0*c(i,2)*3 + (u0*c(i,2))^2*4.5 - u0^2*1.5); % Haut
   fEq(m,2:n-1,i) = t(i); % Bas
   fIn(:,1,i) = fEq(:,1,i); fIn(:,n,i) = fEq(:,n,i);
   fIn(1,2:n-1,i) = fEq(1,2:n-1,i); fIn(m,2:n-1,i) = fEq(m,2:n-1,i);
end

for i = 1 : 9 % Calculer les densités
    rho = rho + fIn(:,:,i);
end

for k = 1 : n % Calculer les vitesses
    for j = 1 : m
        for i = 1 : 9
            ux(j,k) = ux(j,k) + (c(i,2)*fIn(j,k,i));
            uy(j,k) = uy(j,k) + (c(i,1)*fIn(j,k,i));
        end
        ux(j,k) = ux(j,k)/rho(j,k);
        uy(j,k) = uy(j,k)/rho(j,k);
    end
end

for j = 1 : m % Calculer les fEq
    for k = 1 : n
        u2 = ux(j,k)*ux(j,k)+uy(j,k)*uy(j,k);
        for i = 1 : 9
            uc = ux(j,k)*c(i,2)+uy(j,k)*c(i,1);
            fEq(j,k,i) = rho(j,k)*t(i)*(1 + uc*3 + uc^2*4.5 - u2*1.5);
        end
    end
end

for i = 1 : 9 % Calculer tous les fOut
    fOut(:,:,i) = (1 - 1/tau)*fIn(:,:,i) + (1/tau)*fEq(:,:,i);
    fOut(:,:,i) = circshift(fOut(:,:,i), c(i,:));
end